-- vi: et sw=4 ts=4:
-- hsv2
---@module "hsv2.core.packer"
local M = {}
local L = {}

---Init packer and install it if missing
---@param config   table
---@return boolean status
function M.init(config)
    local log = require("hsv2.utils.log").log
    if not packer_plugins then
        local install_path = vim.fn.stdpath('data')
            .. '/site/pack/hsv2/opt/packer.nvim'
        local status, output = L.install(install_path)
        if not status then
            log({
                { "Error: failed to install packer\n", "Error" },
                { output, "Error" },
            }, "err")

            return false
        end
        log("packer installed in " .. install_path, "info")
        vim.cmd("packadd packer.nvim")
    end

    local status, packer = pcall(require, "packer")
    if status then
        config = vim.tbl_extend("force", {}, config, { plugin_package = "hsv2" })
        packer.init(config)
        ---@module "packer"
        M.packer = packer
        log("packer loaded", "info")

        return M.packer
    end

    log({
        { "failed to load packer\n", "Error" },
        { packer, "Error" },
    }, "err")

    return false
end

---Download packer to the given path
---@param path string
---@return boolean, table? status status and command output on error.
function L.install(path)
    if vim.fn.empty(vim.fn.glob(path)) > 0 then
        local log = require("hsv2.utils.log").log
        log("downloading packer…", "info")
        local output = vim.fn.system {
            'git',
            'clone',
            '--depth=1',
            'https://github.com/wbthomason/packer.nvim',
            path
        }

        return vim.v.shell_error == 0, output
    end

    return true
end

return M
