-- vi: et sw=4 ts=4:
-- hsv2
---@module "hsv2.utils.paths"
local M = {}

local str = require("hsv2.utils.str")
local asserts = require("hsv2.utils.asserts")

---@alias path_raw string|number
---@alias path     path_raw[]

---Parse an index number, dot separated keys into a list of keys.
---@param  raw_path? path|path_raw  Path component, or dot separated key names to parse.
---@return path?     path           Returns the path list of keys or nil if none.
function M.parse(raw_path)
    local path_type = asserts.type(raw_path, {
        "string",
        "number",
        "table",
    }, "path", true)

    local path = path_type == "string" and str.split(raw_path, "%.")
        or path_type == "number" and { raw_path }
        or path_type == "table" and raw_path or {}

    return next(path) and path or nil
end

---Returns string representation of a path.
---@param  path     path|path_raw Path to convert to string
---@return string
function M.to_string(path, str, dont_parse_str)
    asserts.type(path, {
        "string",
        "number",
        "table",
    }, "path")
    asserts.type(str, "string", "str", true)

    path = M.parse(path)

    local parsed_str = M.parse(str)
    str = parsed_str and (dont_parse_str
            and (str or "")
            or M.to_string(parsed_str, nil, true)) or ""

    if not path then return str end

    for _, v in ipairs(path) do
        str = str .. (str ~= "" and "." or "")
        str = str .. v
    end

    return str
end

---Compare two path.
---@param  path1   path
---@param  path2   path
---@return boolean true  If paths are equals.
---@return boolean false If paths are not related.
---@return number   1    If path2 is part of path1.
---@return number  -1    If path1 is part of path2.
function M.compare(path1, path2)
    path1 = M.parse(path1) or {}
    path2 = M.parse(path2) or {}
    for i, key in ipairs(path1) do
        if path2[i] ~= key then
            return i > 1 and -1
        end
    end

    return #path2 > #path1 and 1 or true
end

return M
